------------------------------------------------------------------------------
--  This file is a part of the LPP VHDL IP LIBRARY
--  Copyright (C) 2009 - 2013, Laboratory of Plasmas Physic - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 2 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
-------------------------------------------------------------------------------
-- Author : Jean-christophe Pellion
-- Mail   : jean-christophe.pellion@lpp.polytechnique.fr
--          jean-christophe.pellion@easii-ic.com
-------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;

ENTITY lpp_counter_LPP_JCP IS
  
  GENERIC (
    nb_wait_period : INTEGER := 750;
    nb_bit_of_data : INTEGER := 16
    );
  PORT (
    clk   : IN  STD_LOGIC;
    rstn  : IN  STD_LOGIC;
    clear : IN  STD_LOGIC;
    full  : OUT STD_LOGIC;
    data  : OUT STD_LOGIC_VECTOR(nb_bit_of_data-1 DOWNTO 0);
    new_data : OUT STD_LOGIC
    );

END lpp_counter_LPP_JCP;

ARCHITECTURE beh OF lpp_counter_LPP_JCP IS

  SIGNAL counter_wait : INTEGER;
  SIGNAL counter_data : INTEGER;

  SIGNAL new_data_s : STD_LOGIC;
BEGIN  -- beh

  PROCESS (clk, rstn)
  BEGIN  -- PROCESS
    IF rstn = '0' THEN                  -- asynchronous reset (active low)
      counter_wait <= 0;
      counter_data <= 0;
      full         <= '0';
      new_data_s   <= '0';
    ELSIF clk'event AND clk = '1' THEN  -- rising clock edge
      IF clear = '1' THEN
        counter_wait <= 0;
        counter_data <= 0;
        full         <= '0';
        new_data_s   <= NOT new_data_s;
      ELSE
        IF counter_wait = nb_wait_period-1 THEN
          counter_wait <= 0;
          new_data_s <= NOT new_data_s;
          IF counter_data = (2**nb_bit_of_data)-1 THEN
            full         <= '1';
            counter_data <= 0;
          ELSE
            full         <= '0';
            counter_data <= counter_data +1;
          END IF;
        ELSE
          full         <= '0';
          counter_wait <= counter_wait +1;
        END IF;
      END IF;
    END IF;
  END PROCESS;

  data <= STD_LOGIC_VECTOR(to_unsigned(counter_data,nb_bit_of_data));
  new_data <= new_data_s;
  
END beh;
